#!/usr/bin/expectk -file

global spawned ERR RES VOLUME CONNEX path GraphBuilt
global GraphFile
global xScale yScale zScale VtkGraph VtkWS

set xScale 0.3
set yScale 0.3
set zScale 20
set VtkGraph "polyLayer"
set VtkWS "vtkPoly"

set path(size) 0
set GraphFile "graph"
set ERR ""
set RES ""
set VOLUME ""
set CONNEX ""
set spawned 0
set GraphBuilt 0
set timeout -1
log_user 1

catch {exec mkdir planifTrace}
if {![file isdirectory planifTrace]} {
	puts "Can't create planifTrace directory"
	exit
}

proc GraphRead { f } {
	global spawned ERR
	set ERR ""
	if {! $spawned} {set ERR "Not yet spawned";return 0}
	exp_send "Read $f\n"
	expect {
		"OK*> "		{return 1}
		-re "ERROR : (.*)\r?\n" {
			set ERR [string trim $expect_out(1,string)];
			return 0
		}
	}
}

proc GraphSave { f } {
	global spawned ERR
	set ERR ""
	if {! $spawned} {set ERR "Not yet spawned";return 0}
	exp_send "Save $f\n"
	expect {
		"OK*> "		{return 1}
		-re "ERROR : (.*)\r?\n" {
			set ERR [string trim $expect_out(1,string)];
			return 0
		}
	}
}

proc GraphVtkExport { f ws xScale yScale zScale } {
	global spawned ERR
	set ERR ""
	if {! $spawned} {set ERR "Not yet spawned";return 0}
	exp_send "VtkExport $f $ws $xScale $yScale $zScale\n"
	expect {
		"OK*> "		{return 1}
		-re "ERROR : (.*)\r?\n" {
			set ERR [string trim $expect_out(1,string)];
			return 0
		}
	}
}


proc GraphStart { x y theta } {
	global spawned ERR
	set ERR ""
	if {! $spawned} {set ERR "Not yet spawned";return 0}
	exp_send "Start $x $y $theta\n"
	expect {
		"OK*> "		{return 1}
		-re "ERROR : (.*)\r?\n" {
			set ERR [string trim $expect_out(1,string)];
			return 0
		}
	}
}


proc GraphEnd { x y theta } {
	global spawned ERR
	set ERR ""
	if {! $spawned} {set ERR "Not yet spawned";return 0}
	exp_send "End $x $y $theta\n"
	expect {
		"OK*> "		{return 1}
		-re "ERROR : (.*)\r?\n" {
			set ERR [string trim $expect_out(1,string)];
			return 0
		}
	}
}

proc GraphInit { workSpace beacons r a t } {
	global spawned ERR GraphBuilt
	set ERR ""
	if {! $spawned} {set ERR "Not yet spawned";return 0}
	exp_send "Init $workSpace $beacons $r $a $t\n"
	expect {
		"OK*> "		{return 1}
		-re "ERROR : (.*)\r?\n" {
			set ERR [string trim $expect_out(1,string)];
			return 0
		}
	}
	set GraphBuilt 1
}
	

proc GraphPath { } {
	global spawned ERR RES path
	set ERR ""
	if {! $spawned} {set ERR "Not yet spawned";return 0}
	exp_send "Path\n"
	set motif "Path length : (\[0-9\]+)\[^O\]*OK.*> "
	expect {
		-re $motif {
			set path(size) $expect_out(1,string)
		}
		-re "ERROR : (.*)\r?\n" {
			set ERR [string trim $expect_out(1,string)];
			return 0
		}
		timeout	{puts "Time out in path length"}
	}
	
	set motif "(Theta\[^O\]+)OK.*> "
	for {set i 0} {$i < $path(size)} {incr i} {
		exp_send "Step $i\n"
		expect {
			-re $motif {
				set path($i) $expect_out(1,string)
			}
			-re "ERROR : (.*)\r?\n" {
				set ERR [string trim $expect_out(1,string)];
				return 0
			}
			timeout	{ puts "Time out in path step $i" }
		}
	}
	return 1
}

proc GraphVolume { } {
	global spawned ERR RES
	set ERR ""
	if {! $spawned} {set ERR "Not yet spawned";return 0}
	exp_send "Volume\n"
	set motif "\r?\n(\[^\r\n\]*)\r?\nOK.*> " 
	expect {
		-re $motif {set RES $expect_out(1,string);return 1}
		-re "ERROR : (.*)\r?\n" {
			set ERR [string trim $expect_out(1,string)];
			return 0
		}
	}
	return 0
}

proc GraphConnex { } {
	global spawned ERR RES
	set ERR ""
	if {! $spawned} {set ERR "Not yet spawned";return 0}
	exp_send "Connex\n"
	set motif "\r?\n(\[^\r\n\]*)\r?\nOK.*> " 
	expect {
		-re $motif {set RES $expect_out(1,string);return 1}
		-re "ERROR : (.*)\r?\n" {
			set ERR [string trim $expect_out(1,string)];
			return 0
		}
	}
	return 0
}

proc Quit { } {
	global spawned 
	if {$spawned} {
		exp_send "Quit\n"
		expect "OK\r?\n"
	}
	exit
}

proc Help { } {
	global spawned 
	if {! $spawned} {return}
	exp_send "Help\n"

	expect -re "\n(.*)\n> "
	puts $expect_out(1,string)
}



global width height r_canvas ExportIdx
global x_position y_position 
global baliseName polysName stateFile
global action lastPoint initPoint Status
global GRID PORTEE ALPHA SPEEDY THETA_STEP
global startb endb thetaOk xyOk
global xStart yStart thetaStart
global xEnd yEnd thetaEnd
global step



# arrays
global balises obstacles
global environnement environnementSaved
global newPolygon

set width 600
set height 400
set x_position ""
set y_position ""
set ExportIdx 0

proc Init {} {
	global stroke balises obstacles
	global environnement action GRID ALPHA PORTEE THETA_STEP
	global xStart yStart thetaStart
	global xEnd yEnd thetaEnd VOLUME ERR RES
	set stroke(N) 0
	set balises(N) 0
	set obstacles(N) 0
	set environnement {}
	set action 1
	set GRID 5
	set ALPHA 180
	set PORTEE 500
	set THETA_STEP 5
	set xStart 0
	set yStart 0
	set thetaStart 0
	set xEnd 0
	set yEnd 0
	set thetaEnd 0
	set VOLUME ""
	set CONNEX ""
	set ERR ""
	set REs ""
}
Init

set baliseName "beacons"
set polysName "workspace"
set stateFile "state"
set scriptVisu "go"


proc DrawBeacon { P t color } {
	global r_canvas 
	set x [expr int([lindex $P 0])]
	set y [expr int([lindex $P 1])]
	set pa [list [expr $x - 6] [expr $y - 6]]
	set pb [list [expr $x + 7] [expr $y + 7]]
	set p0 [list [expr $x - 3] $y]
	set p1 [list [expr $x + 4] $y]
	set p2 [list $x [expr $y - 3]]
	set p3 [list $x [expr $y + 4]]
	eval "$r_canvas create oval $P $P -tag $t -outline $color "
	eval "$r_canvas create oval $pa $pb -tag $t -outline $color "
}



proc DrawFilledPoly { poly t color } {
	global r_canvas 
	eval "$r_canvas create poly [join $poly] -tag $t -fill $color"
}

proc DrawPoly { poly t color } {
	global r_canvas 
	eval "$r_canvas create line [join $poly] -tag $t -fill $color"
	eval "$r_canvas create line [lindex $poly 0] \
		[lindex $poly [expr [llength $poly]-1]] -tag $t -fill $color"
}

proc DrawRobot { x y t SE} {
	global r_canvas
	set col(Start) blue
	set col(End) green
	set col(Step) cyan
	if {"$SE" != "Step"} {catch {$r_canvas delete "robot$SE"}}
	set P1 [list [expr $x - 6] [expr $y - 6]]
	set P2 [list [expr $x + 6] [expr $y + 6]]
	eval "$r_canvas create oval $P1 $P2 -tag robot$SE -fill $col($SE)"
	set P3 [list [expr $x + cos($t)*20] [expr $y + sin($t)*20]]
	eval "$r_canvas create line $x $y $P3 -tag robot$SE -fill $col($SE) -width 2"
}

proc Refresh { } {
	global r_canvas 
	global balises environnement obstacles
	global xStart yStart thetaStart
	global xEnd yEnd thetaEnd
	eval "$r_canvas delete all"
	if {[llength $environnement] > 0} {
		DrawPoly $environnement environnement green
	}
	for {set i 0} {$i < $obstacles(N)} {incr i} {
		DrawFilledPoly $obstacles($i) "obstacles($i)" grey
		DrawPoly $obstacles($i) "obstacles($i)" blue
	}
	for {set i 0} {$i < $balises(N)} {incr i} {
		DrawBeacon $balises($i) "balises($i)" "#D00000"
	}
	DrawRobot $xStart $yStart $thetaStart "Start"
	DrawRobot $xEnd $yEnd $thetaEnd "End"
}


proc PrintPoly { fid poly hole } {
	puts $fid [llength $poly]
	puts $fid $hole
	foreach pts $poly {
		puts $fid "$pts"
	}
}

proc RestoreState { } {
	global balises environnement obstacles stateFile PORTEE ALPHA
	global GraphFile RES GraphBuilt VOLUME CONNEX
	UnbindAll
	set fid [open $stateFile "r"]
	set line [gets $fid]
	scan $line "Portee : %d" PORTEE
	set line [gets $fid]
	scan $line "Alpha : %d" ALPHA
# Balises :
	set line [gets $fid]
	scan $line "Balises : %d" n
	catch {unset balises}
	set balises(N) $n
	for {set i 0} {$i < $n} {incr i} {
		set line [gets $fid]
		scan $line " %f %f " x y
		set balises($i) [list $x $y]
	}

# Workspace :
	set line [gets $fid] 
	set n [expr [string trim [gets $fid]] - 1] 
	set res [ReadPoly $fid]
	set environnement [lindex $res 1]
	catch {unset obstacles}
	set obstacles(N) $n
	
	for {set i 0} {$i < $n} {incr i} {
		set res [ReadPoly $fid]
		set obstacles($i) [lindex $res 1]
	}
# Graph Built
	set line [gets $fid]
	scan $line "GraphBuilt = %d" GraphBuilt
	close $fid

	if {$GraphBuilt} { 
		GraphRead $GraphFile 
		if {[GraphVolume]} {set VOLUME $RES}
		if {[GraphConnex]} {set CONNEX $RES}
	}

}

proc RestoreSecure { } {
	UnbindAll
	if [catch RestoreState result] {
		puts stderr "Error during restoration"
		puts stderr $result
		Init
	}
	SelectAndMove
}

proc SaveState { } {
	global balises environnement obstacles stateFile PORTEE ALPHA GraphBuilt
	global GraphFile
	UnbindAll
	set fid [open $stateFile "w"]
	puts $fid "Portee : $PORTEE"
	puts $fid "Alpha : $ALPHA"
	puts $fid "Balises : $balises(N)"
	for {set i 0} {$i < $balises(N)} {incr i} {
		puts $fid "$balises($i)"
	}

	puts $fid "WorkSpace : "
	puts $fid [expr 1 + $obstacles(N)]
	PrintPoly $fid $environnement 0
	for {set i 0} {$i < $obstacles(N)} {incr i} {
		PrintPoly $fid $obstacles($i) 1
	}
	puts $fid "GraphBuilt = $GraphBuilt"
	close $fid
	if {$GraphBuilt} {GraphSave "graph"}
	SelectAndMove
}

proc Print { } {
	global balises environnement obstacles polysName baliseName
	UnbindAll
	set fid [open $baliseName "w"]
	puts $fid "$balises(N)"
	for {set i 0} {$i < $balises(N)} {incr i} {
		puts $fid "$balises($i)"
	}
	close $fid

	set fid [open $polysName "w"]
	puts $fid [expr 1 + $obstacles(N)]
	PrintPoly $fid $environnement 0
	for {set i 0} {$i < $obstacles(N)} {incr i} {
		PrintPoly $fid $obstacles($i) 1
	}
	close $fid
	SelectAndMove
}

proc ReadPoly { fid } {
	set n [string trim [gets $fid]]
	set hole [string trim [gets $fid]]
	set pts {}
	for {set i 0} {$i < $n} {incr i} {
		set line [gets $fid]
		scan $line " %f %f " x y
		lappend pts [list $x $y]
	}
	return [list $hole $pts]
}


proc Read { } {
	global balises environnement obstacles polysName baliseName GraphBuilt
	set fid [open $baliseName]
	set n [string trim [gets $fid]]
	catch {unset balises}
	set balises(N) $n
	for {set i 0} {$i < $n} {incr i} {
		set line [gets $fid]
		scan $line " %f %f " x y
		set balises($i) [list $x $y]
	}
	close $fid

	set fid [open $polysName]
	set n [expr [string trim [gets $fid]] - 1] 
	set res [ReadPoly $fid]
	set environnement [lindex $res 1]
	catch {unset obstacles}
	set obstacles(N) $n
	
	for {set i 0} {$i < $n} {incr i} {
		set res [ReadPoly $fid]
		set obstacles($i) [lindex $res 1]
	}
	close $fid
	set GraphBuilt 0
	
}

proc ReadSecure { } {
	global baliseName polysName GraphBuilt
	UnbindAll
	if [catch {Read} result] {
		puts stderr "Error during reading $baliseName and $polysName"
		Init
	}
	SelectAndMove
	set GraphBuilt 0
}


proc cgrid { x y } {
	global GRID
	if {$GRID <= 1} {return [list $x $y]}
	set x [expr round($x / $GRID)*$GRID]
	set y [expr round($y / $GRID)*$GRID]
	return [list $x $y]
}

proc UnbindAll { } {
	global r_canvas
	bind $r_canvas <Motion> {}
	bind $r_canvas <ButtonRelease-1> {}
	bind $r_canvas <ButtonRelease-2> {}
	bind $r_canvas <ButtonRelease-3> {}
	bind $r_canvas <Button-1> {}
	bind $r_canvas <B1-Motion> {}
	Refresh
}

proc markObject { t } {
	global r_canvas
	if {[string match "balises*" $t]} {
		$r_canvas itemconfigure $t -outline white
	} else {
		$r_canvas itemconfigure $t -fill white
	}
}

proc SelectAndKill { } {
	global r_canvas action Status
	set action 2
	set Status "Select an object to destroy (left button)."
	UnbindAll
	bind $r_canvas <ButtonRelease-1> {HookObjectForKill %x %y}
}

proc HookObjectForKill { x y } {
	global r_canvas lastPoint initPoint Status
	set obj [$r_canvas find closest $x $y 10]
	if {"$obj" == ""} { return }
	set t [lindex "[$r_canvas gettags $obj]" 0 ]
	if {[string match "robot*" $t]} {return}
	set Status "Confirm destruction by middle button. Right to cancel."
	markObject $t
	bind $r_canvas <ButtonRelease-2> [list KillConfirm $t]
	bind $r_canvas <ButtonRelease-3> {SelectAndMove}
}

proc KillConfirm { t } {
	global balises obstacles environnement GraphBuilt
	if {[string match "environnement" $t]} {
		set GraphBuilt 0
		set environnement {}
		SelectAndMove
		return
	}
	if {[string match "obstacles*" $t]} {
		set GraphBuilt 0
		set n [expr $obstacles(N) - 1]
		incr obstacles(N) -1
		set $t $obstacles($n)
		SelectAndMove
		return
	}
	if {[string match "balises*" $t]} {
		set GraphBuilt 0
		set n [expr $balises(N) - 1]
		incr balises(N) -1
		set $t $balises($n)
		SelectAndMove
		return
	}
}

proc SelectAndMove { } {
	global r_canvas action Status
	set action 1
	set Status "Click to select any object (left button)."
	UnbindAll
	bind $r_canvas <ButtonRelease-1> {HookObjectForMove %x %y}
}

proc sub {P Q} {
	return [cgrid [expr [lindex $Q 0] - [lindex $P 0]] \
		       [expr [lindex $Q 1] - [lindex $P 1]]]
}

proc add {P Q} {
	return [cgrid [expr [lindex $Q 0] + [lindex $P 0]] \
		       [expr [lindex $Q 1] + [lindex $P 1]]]
}


proc MoveBalise { t P } {
	global balises
	set old $$t
	eval set old $old
	return [add $old $P]
}

proc MovePoly { t P } {
	global environnement obstacles
	set pts {}
	set old $$t
	eval set old $old
	foreach A $old {
		lappend pts [add $A $P]
	}
	return $pts
}
	


proc HookObjectForMove { x y } {
	global r_canvas lastPoint initPoint Status
	set obj [$r_canvas find closest $x $y 10]
	if {"$obj" == ""} { return }
	set t [lindex "[$r_canvas gettags $obj]" 0 ]
	if {[string match "robot*" $t]} {return}
	if {[string match "balises*" $t]} {
		set drawfunc "DrawBeacon"
		set movefunc "MoveBalise"
	} else {
		set drawfunc "DrawPoly"
		set movefunc "MovePoly"
	}
	set lastPoint [list $t $drawfunc $movefunc]
	set initPoint [list $x $y]
	set Status "Left again to release selected object. Right to cancel movement."
	markObject $t
	bind $r_canvas <Motion> {MoveObject %x %y}
	bind $r_canvas <ButtonRelease-1> {ReleaseObject %x %y}
	bind $r_canvas <ButtonRelease-3> {SelectAndMove}
}


proc MoveObject { x y } {
	global r_canvas lastPoint initPoint
	set deplacement [sub $initPoint [cgrid $x $y]]
	set dfunc [lindex $lastPoint 1]
	set mfunc [lindex $lastPoint 2]
	set t [lindex $lastPoint 0]
	$r_canvas delete $t
	$dfunc [$mfunc $t $deplacement] $t white
	set GraphBuilt 0
	

}
	
proc ReleaseObject { x y } {
	global r_canvas lastPoint initPoint
	global balises obstacles environnement
	set deplacement [sub $initPoint [cgrid $x $y]]
	set t "[lindex $lastPoint 0]"
	if {[string match "balises*" "$t"]} {
		set $t [MoveBalise "$t" $deplacement]
	} else {
		set $t [MovePoly "$t" $deplacement]
	}
	SelectAndMove
}
		
		


proc NewBeacon { } {
	global r_canvas balises action Status
	set action 5
	set Status "Left button to put a beacon. Left again to validate. Right to cancel."
	UnbindAll
	bind $r_canvas <ButtonRelease-1> {NBPutBeacon %W %x %y}
	bind $r_canvas <ButtonRelease-3> {NBCancel }
	bind $r_canvas <Motion> {}
}

proc NBDrawBeacon {W x y} {
	global r_canvas
	catch { eval "$r_canvas delete temp" }
	DrawBeacon [cgrid $x $y] temp white
}

proc NBPutBeacon {W x y} {
	global r_canvas
	NBDrawBeacon $W $x $y
	bind $r_canvas <Motion> {NBDrawBeacon %W %x %y}
	bind $r_canvas <ButtonRelease-1> {NBAddBeacon %W %x %y}
}

	
proc NBCancel { } {
	SelectAndMove
}

proc NBAddBeacon {W x y} {
	global balises GraphBuilt
	set balises($balises(N)) [cgrid $x $y]
	incr balises(N)
	set GraphBuilt 0
	NewBeacon
}
	

proc NewObstacle { } {
	global r_canvas newPolygon lastPoint action Status
	set Status "Left button to add vertices. Middle for last one. Right to cancel. Polygon MUST be simple !"
	set action 4
	set newPolygon {}
	set lastPoint "" 
	UnbindAll
	bind $r_canvas <ButtonRelease-1> {PolyAddPoint %W %x %y}
	bind $r_canvas <ButtonRelease-2> {ObstacleAddLastPoint %W %x %y}
	bind $r_canvas <ButtonRelease-3> {PolyCancel}
	bind $r_canvas <Motion> {}
}

	

proc ObstacleAddLastPoint { W x y } {
	global newPolygon obstacles GraphBuilt
	PolyAddPoint $W $x $y
	set obstacles($obstacles(N)) $newPolygon
	catch {unset newPolygon}
	incr obstacles(N)
	set GraphBuilt 0
	NewObstacle
}




proc NewWorkSpace { } {
	global r_canvas newPolygon lastPoint action Status
	set action 3
	set Status "Left button to add vertices. Middle for last one. Right to cancel. Polygon MUST be simple !"
	catch { eval "$r_canvas itemconfigure environnement -fill grey" }
	set newPolygon {}
	set lastPoint "" 
	UnbindAll
	bind $r_canvas <ButtonRelease-1> {PolyAddPoint %W %x %y}
	bind $r_canvas <ButtonRelease-2> {WSAddLastPoint %W %x %y}
	bind $r_canvas <ButtonRelease-3> {PolyCancel}
	bind $r_canvas <Motion> {}
}

proc PolyCancel { } {
	catch {unset newPolygon}
	Refresh
	SelectAndMove
}

proc PolyMove { W x y } {
	global r_canvas lastPoint
	catch {eval "$r_canvas delete temp"}
	set current [cgrid $x $y]
	eval "$r_canvas create line $lastPoint $current -fill white -tag temp"
}

proc PolyAddPoint { W x y } {
	global r_canvas newPolygon lastPoint
	catch {eval "$r_canvas delete temp"}
	set current [cgrid $x $y]
	if { "$lastPoint" != "" } {
		eval "$r_canvas create line $lastPoint $current -fill white -tag env"
	}
	set lastPoint $current
	lappend newPolygon $lastPoint
	bind $r_canvas <Motion> {PolyMove %W %x %y}
}

proc WSAddLastPoint { W x y } {
	global newPolygon environnement GraphBuilt
	PolyAddPoint $W $x $y
	set environnement $newPolygon
	set GraphBuilt 0
	unset newPolygon
	SelectAndMove
}


proc BuildGraph { } {
	global baliseName polysName GraphBuilt
	global PORTEE ALPHA THETA_STEP
	global VOLUME RES ERR CONNEX

	Print 
	set a [expr $ALPHA * 3.1415927 / 180.0]

	set ERR ""
	set RES ""
	set res [GraphInit $polysName $baliseName $PORTEE $a $THETA_STEP]
	if {[GraphVolume]} {set VOLUME $RES}
	if {[GraphConnex]} {set CONNEX $RES}
	set GraphBuilt 1
	SelectAndMove
}

proc Visualize { } {
	global xScale yScale zScale VtkGraph VtkWS
	set res [GraphVtkExport $VtkGraph $VtkWS $xScale $yScale $zScale]
	if {$res} {
		catch {exec ./polyPlot $VtkGraph $VtkWS}
	}
}
	

proc Export { } {
	global r_canvas width height ExportIdx
	set name [join [list "planifTrace/workspace" \
		[format "%03d" $ExportIdx] ".eps"] ""]
	while {[file exists $name]} {
		incr ExportIdx
		set name [join [list "planifTrace/workspace" \
			[format "%03d" $ExportIdx] ".eps"] ""]
	}
	$r_canvas postscript -file "$name" \
	-x 0 -y 0 -width $width -height $height
}

proc DrawRobotTheta { x y SE} {
	global xStart yStart xEnd yEnd
	eval "set t [expr atan2(\$y-\$y$SE,\$x-\$x$SE)]"
	eval "DrawRobot \$x$SE \$y$SE $t $SE"
}
	
proc Recordxy { x y SE} {
	global xyOk xStart yStart xEnd yEnd
	eval "set x$SE \$x"
	eval "set y$SE \$y"
	set xyOk 1
}

proc RecordTheta { x y SE} {
	global thetaOk xStart yStart xEnd yEnd thetaStart thetaEnd
	eval "set theta$SE [expr atan2(\$y-\$y$SE,\$x-\$x$SE)]"
	set thetaOk 1
}

proc StartPoint { } {
	global startb thetaOk xyOk r_canvas Status
	global xStart yStart thetaStart
	$startb configure -relief sunken
	set Status "Left button to choose robot position"
	set thetaOk 0
	set xyOk 0
	bind $r_canvas <Motion> {DrawRobot %x %y 0 "Start"}
	bind $r_canvas <ButtonRelease-1> {Recordxy %x %y "Start"}
	tkwait variable xyOk
	set Status "Left button to choose robot orientation"
	bind $r_canvas <Motion> {DrawRobotTheta %x %y "Start"}
	bind $r_canvas <ButtonRelease-1> {RecordTheta %x %y "Start"}
	tkwait variable thetaOk
	UnbindAll

	GraphStart $xStart $yStart $thetaStart

	$startb configure -relief raised
	SelectAndMove
}
	

	

proc EndPoint { } {
	global endb thetaOk xyOk r_canvas
	global xEnd yEnd thetaEnd
	$endb configure -relief sunken
	set thetaOk 0
	set xyOk 0
	set Status "Left button to choose robot position"
	bind $r_canvas <Motion> {DrawRobot %x %y 0 "End"}
	bind $r_canvas <ButtonRelease-1> {Recordxy %x %y "End"}
	tkwait variable xyOk
	set Status "Left button to choose robot orientation"
	bind $r_canvas <Motion> {DrawRobotTheta %x %y "End"}
	bind $r_canvas <ButtonRelease-1> {RecordTheta %x %y "End"}
	tkwait variable thetaOk
	UnbindAll

	GraphEnd $xEnd $yEnd $thetaEnd

	$endb configure -relief raised
	SelectAndMove
}

proc DrawStep { dummy } {
	global r_canvas path step
	catch {$r_canvas delete robotStep}
	set theta [lindex $path($step) 0]
	set poly [lindex $path($step) 1]
	set balises [lindex $path($step) 2]

	DrawFilledPoly $poly robotStep blue
	for {set i 0} {$i < [llength $balises]} {incr i} {
		DrawBeacon [lindex $balises $i] robotStep cyan
	}
	set x 0
	set y 0
	for {set i 0} {$i < [llength $poly]} {incr i} {
		set x [expr $x + [lindex [lindex $poly $i] 0]]
		set y [expr $y + [lindex [lindex $poly $i] 1]]
	}
	set x [expr $x / [llength $poly]]
	set y [expr $y / [llength $poly]]
	DrawRobot $x $y $theta Step
}


proc ComputePath { } {
	global path step
	if {![GraphPath]} return;
	set fid [open "path" "w"]
	puts $fid "Size : "
	puts $fid $path(size)
	for {set i 0} {$i < $path(size)} {incr i} {
		puts $fid "\nStep $i"
		puts $fid "$path($i)"
		set l [split $path($i) "\n"]
		scan [lindex $l 0] " Theta = %e " theta
		scan [lindex $l 3] " %d " n
		set pts {}
		for {set j 5} {$j < [expr $n + 5]} {incr j} {
			scan [lindex $l $j] " %e %e " x y
			lappend pts [list $x $y] 
		}
		set v [split [lindex $l [expr $n + 5]] "\{"]
		set vis {}
		for {set j 1} {$j < [llength $v]} {incr j} {
			scan [lindex $v $j] "%e,%e" x y
			lappend vis [list $x $y]
		}
		set path($i) [list $theta $pts $vis]
		puts $fid "$path($i)"
	}
	close $fid
	
	set step 0
	wm withdraw .
	toplevel .path
	frame .path.b
	button .path.ok -text "OK" -command {destroy .path}
	button .path.exp -text "Export"   -command Export
	scale .path.step -variable step -from 0 -to [expr $path(size)-1] \
		-length 200 -width 5 -command DrawStep -orient horizontal
	button .path.b.p -text "prev." -command \
		{if {$step>0} {set step [expr $step - 1];DrawStep 0}}
	button .path.b.n -text "next." -command \
		{if {$step<[expr $path(size)-1]} {set step [expr $step + 1];DrawStep 0}}
	pack .path.b.p .path.b.n -side left
	pack .path.b .path.step .path.exp .path.ok -side top

	tkwait window .path
	wm deiconify .
	Refresh
	
}

proc InitDisplay { } {
	global width height r_canvas polysName baliseName action
	global startb endb
	toplevel .a
	frame .b
	frame .c
	frame .d
	frame .e
	frame .f
	frame .g -relief ridge -borderwidth 2
	frame .g.a
	frame .g.b

	label .f.lp -text "Range "
	scale .f.p -variable PORTEE -from 0 -to 1000 -orient horiz -width 10 \
		-resolution 50 -tickinterval 200
	label .f.la -text "Ouverture "
	scale .f.a -variable ALPHA -from 0 -to 360 -orient horiz -width 10 \
		-resolution 15 -tickinterval 60

	
	label .b.lb -text "Beacon file"
	entry .b.b -textvariable baliseName -width 30
	label .b.lp -text "Workspace file"
	entry .b.p -textvariable polysName -width 30

	radiobutton .d.mv -text "Select & move" -variable action -value 1 \
		-relief groove -command SelectAndMove
	radiobutton .d.kl -text "Select & kill" -variable action -value 2 \
		-relief groove -command SelectAndKill
	radiobutton .d.nw  -text "New workspace" -variable action -value 3 \
		-relief groove -command NewWorkSpace
	radiobutton .d.no -text "New obstacle" -variable action -value 4 \
		-relief groove -command NewObstacle
	radiobutton .d.nb -text "New beacon" -variable action -value 5 \
		-relief groove -command NewBeacon
	
	set r_canvas [canvas .a.cv -width $width -height $height -background black]
	$r_canvas config -cursor {tcross white black}

	label .c.lg -text "grid"
	scale .c.g -variable GRID -from 0 -to 20 -orient horiz -width 10 -resolution 5 \
		-tickinterval 10
	button .c.z -text "Reset" -command  {Init ; Refresh }
	button .c.r -text "Load State" -command RestoreSecure
	button .c.p -text "Save State" -command SaveState
	button .c.x -text "Export"   -command Export
	button .c.v -text "Visualize"   -command Visualize
	button .c.e -text "Exit" -command Quit

	label  .g.a.l -text "Graph operations : "
	button .g.a.i -text "Build graph" -command BuildGraph
	set startb [button .g.a.s -text "Start point" -command StartPoint]
	set endb [button .g.a.e -text "End point"   -command EndPoint]
	button .g.a.c -text "Compute path" -command ComputePath
	

	entry .g.b.v -textvariable VOLUME -state disabled -relief flat -width 16
	entry .g.b.c -textvariable CONNEX -state disabled -relief flat -width 15
	label .g.b.l -text "Error :"
	entry .g.b.e -textvariable ERR -state disabled -relief flat -width 50
	

	label .e.l -text "What to do :"
	entry .e.e -textvariable Status -state disabled -relief flat -width 80
	
	pack .d.mv .d.kl .d.nw .d.no .d.nb -side left -fill x -expand true
	pack .b.lb .b.b .b.lp .b.p -side left -fill x -expand true
	pack .a.cv -side right -fill x -expand true
	pack .c.r .c.p .c.z .c.x .c.v .c.e .c.lg .c.g -side left  -expand true
	pack .g.a.l -side left 
	pack .g.a.i .g.a.s .g.a.e .g.a.c -side left -padx 15
	pack .e.l .e.e  -side left -fill x  -expand true
	pack .g.b.v .g.b.c .g.b.l -side left 
	pack .g.b.e  -side left -fill x  -expand true
	pack .f.lp -side left 
	pack .f.p -side left -fill x  -expand true
	pack .f.la -side left
	pack .f.a -side left -fill x  -expand true
	pack .g.a .g.b -side top -fill x
	pack .b .d .f .e .c .g -side top -fill x
	SelectAndMove

}

InitDisplay

spawn planifCnx
set spawned 1

expect "> "

expect_after timeout {
	global ERR
	set ERR "Time out"
	puts "Time out"
}
